#include "ch32f20x_conf.h"
#include "bsp.h"
#include "types.h"
#include "debug.h"
#include "sm_timer.h"

//#include "UART.h"


u8 key_event;

u32 time_ms = 0;
void timer_isr();
void key_scan();


//for dbg
DBG_LEVEL dbg_level=DBG_DEBUG;
void DBG_setlevel(DBG_LEVEL level)
{  
    dbg_level=level;
}


u32 bsp_get_time_ms()
{
    return time_ms;
}


void uart_init()
{
    // debug uart
    
    GPIO_InitTypeDef GPIO_InitStructure;
    USART_InitTypeDef USART_InitStructure;
    NVIC_InitTypeDef  NVIC_InitStructure;
    
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1|RCC_APB2Periph_GPIOA, ENABLE);   
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);    

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9; 
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;    
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10; 
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;    
    GPIO_Init(GPIOA, &GPIO_InitStructure);



    USART_InitStructure.USART_BaudRate = 921600;//2000000;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Tx ;//| USART_Mode_Rx;

    USART_Init(USART1, &USART_InitStructure); 
    USART_ITConfig( USART1, USART_IT_RXNE, ENABLE );

    NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init( &NVIC_InitStructure );

    USART_Cmd(USART1, ENABLE);      
    DBG_print(DBG_DEBUG,"uart_init");
}

void timer_init()
{
    TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure={0};
    
    NVIC_InitTypeDef NVIC_InitStructure;
    /* set timer to 1ms */
    RCC_APB1PeriphClockCmd( RCC_APB1Periph_TIM2, ENABLE );

    TIM_TimeBaseInitStructure.TIM_Period = 10000-1;     // 10us * 10000 = 100ms
    TIM_TimeBaseInitStructure.TIM_Prescaler = 480-1; // 48M/480 = 100K
    TIM_TimeBaseInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
    TIM_TimeBaseInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInit( TIM2, &TIM_TimeBaseInitStructure );
    
		TIM_ClearFlag(TIM2, TIM_IT_Update);
    TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);    
		TIM_Cmd(TIM2, ENABLE);
    
		NVIC_SetPriorityGrouping(NVIC_PriorityGroup_0);
		NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
		NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
		NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
		NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
		NVIC_Init(&NVIC_InitStructure);
}

void timer_isr()
{
    time_ms++;
    if(time_ms%50 == 0)
    {
        key_scan();
    }
}

void gpio_init()
{
    GPIO_InitTypeDef  GPIO_InitStructure = {0};
    
    RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOA, ENABLE );
    RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOB, ENABLE );    
    RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOC, ENABLE );    
    RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOD, ENABLE );

    //PA0,1 => RP1,2  , ADC input
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0|GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    //PA5 =>PWRCTL

    //PA6 =>WS2812

    //PA7 =>OPAO

    //PB10,PB11 =>TX3,RX3
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10; 
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;    
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11; 
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;    
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    //PB12 =>US_RX detect

    //PB13 =>selfpwr

    //PA8=> PWMout
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init( GPIOA, &GPIO_InitStructure );

    //PA9,10 =>UART

    //PA11,12=> test_point

    //PA13,14=> SWD

    // PA15,PB3 =>LED1,2
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init( GPIOA, &GPIO_InitStructure );
    // PB4,5,6 => SW3,2,1

    
    
    // conn led  and key.
    // the pin led and key is Multiplexing, key need scan every 10ms.
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init( GPIOC, &GPIO_InitStructure );
    //BSP_CONN_OFF();

    // power switch 3.3 or 5V  and #CLK pin
    
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13|GPIO_Pin_15;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init( GPIOC, &GPIO_InitStructure );


}




void BSP_Init(void)
{
    uart_init();
    //gpio_init();
    timer_init();    
}



u8 key_event;
void key_scan()
{    
    u8 key_sta,led_sta;
    static u16 long_key_cnt;
    static u8 last_key_sta;

    led_sta = GPIOC->OUTDR&2?1:0;
    GPIOC->BSHR = 2;    
    key_sta = (GPIOC->INDR&2)?1:0;

    if(!led_sta)
        GPIOC->BCR = 2;

    if(last_key_sta != key_sta)
    {
        last_key_sta = key_sta;
        if(key_sta)
        {
            if(key_event != KEYEVENT_LONG)
                key_event = KEYEVENT_SHORT;// short key
            long_key_cnt = 0;
        }
    }

    if(key_sta==0)
    {
        if(long_key_cnt<20)
        {
            long_key_cnt++;
        }
        else
        {
            long_key_cnt = 0;
            key_event = KEYEVENT_LONG;// long key
        }
    }
}

void bsp_sys_reboot()
{
    DBG_print(DBG_DEBUG,"bsp_sys_reboot");

    NVIC_SystemReset();
}



void TIM2_IRQHandler(void)
{
    if (TIM_GetITStatus(TIM2, TIM_IT_Update) != RESET)
    {
        timer_isr();
        TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
    }
}

void USART1_IRQHandler( void )
{
    if( USART_GetITStatus( USART1, USART_IT_RXNE ) != RESET )
    {
        u8 c = USART_ReceiveData( USART1 );        
    }
}


