#include "common.h"
#include "debug.h"
#include "types.h"
#include <string.h>
#include <stdio.h>
#include <math.h>



bool common_check_domainstr(const char * domain,u16 len)
{
    u16 i;
    u16 domain_len = strlen(domain);
    
    if(domain_len > len)
    {
        DBG_printf(DBG_DEBUG,"check strlen fail strlen=%d,len=%d\r\n",domain_len,len);
        return FALSE;
    }
    
    for(i=0;i<domain_len;i++)
    {
        if(domain[i] < '.' ||( domain[i] > '9' && domain[i] < 'A') ||\
            (domain[i] > 'Z' && domain[i] < 'a' ) || domain[i] > 'z')
        {

            DBG_printf(DBG_DEBUG,"check domain fail %s\r\n",domain);
            return FALSE;
        }
    }

    return TRUE;
}



void common_strip_2_byteip(const char *str_ip,u8 *bip)
{
    int ip[4];
    sscanf(str_ip,"%d.%d.%d.%d",&ip[0],&ip[1],&ip[2],&ip[3]);
    bip[0]=ip[0];
    bip[1]=ip[1];
    bip[2]=ip[2];
    bip[3]=ip[3];
}



u16 common_crc16(u8* data, u16 len)
{
    const u16 CRCTalbeAbs[] = {
        0x0000, 0xCC01, 0xD801, 0x1400, 0xF001, 0x3C00, 0x2800, 0xE401, 
        0xA001, 0x6C00, 0x7800, 0xB401, 0x5000, 0x9C01, 0x8801, 0x4400, 
    };
    u16  wCRC = 0xFFFF;
    u16  i;
    u8 chChar;

    for (i = 0; i < len; i++)
    {
        chChar = *data++;
        wCRC = CRCTalbeAbs[(chChar ^ wCRC) & 15] ^ (wCRC >> 4);
        wCRC = CRCTalbeAbs[((chChar >> 4) ^ wCRC) & 15] ^ (wCRC >> 4);
    }
    return wCRC;
}


u16 common_crc16_v(u16 init_v, u8* data, u32 len)
{
    const u16 CRCTalbeAbs[] = {
        0x0000, 0xCC01, 0xD801, 0x1400, 0xF001, 0x3C00, 0x2800, 0xE401, 
        0xA001, 0x6C00, 0x7800, 0xB401, 0x5000, 0x9C01, 0x8801, 0x4400, 
    };
    u16  wCRC = init_v;
    u32  i;
    u8 chChar;

    for (i = 0; i < len; i++)
    {
        chChar = *data++;
        wCRC = CRCTalbeAbs[(chChar ^ wCRC) & 15] ^ (wCRC >> 4);
        wCRC = CRCTalbeAbs[((chChar >> 4) ^ wCRC) & 15] ^ (wCRC >> 4);
    }
    return wCRC;
}


u16 ChkSum16(u8* data, u16 len)
{
    u16 chksum = 0;
    u16 i;

    for (i = 0; i < len; i++)
    {
        chksum += data[i] ;
    }
    return chksum;
}



int common_check_hexchar(char  c)
{
    if(!((c>='0'&&c<='9') ||(c>='A' && c<='F') ||(c>='a' && c<= 'f')))
    {
         return (-1);   
    }
    return 0;
}
// һַת һ hex ںϲ
u8 common_char2hex(char c)
{
    u8 hex;

    if(c>='0'&&c<='9') 
        hex = c - '0';
    else if(c>='A' && c<='F')
        hex = c - 'A' +10;
    else if(c>='a' && c<='f')
        hex = c - 'a' +10;
    else
        hex = 0;    
    return hex;
}

/*used to convert the hex string to hex bytes,like "821C220A" -> 0x82,0x1C,0x22,0x0A*/

u32 common_string2hex(char * string, char* hex,u32 maxbytes)
{
    u32 i,string_len,hex_i;
    u8 hex_h,hex_l,l_flag,h_flag;
    hex_l = 0;
    hex_h = 0;
    l_flag = 0;
    h_flag = 0;
    hex_i =  0;
    string_len = strlen(string);
    for(i=0;i<string_len;i++)
    {
        if(hex_i >= maxbytes)
            break;
       
        if(common_check_hexchar(string[i])==0)
        {
            if(h_flag == 0)
            {
                h_flag = 1;
                hex_h= common_char2hex(string[i]);
            }
            else if(l_flag == 0)
            {
                l_flag = 1;
                hex_l = common_char2hex(string[i]);
                hex[hex_i] = (hex_h<<4) + hex_l;
                hex_i ++;
                l_flag = 0;
                h_flag = 0;
            }
        }
        else if(string[i] == ' ')
        {
            if(h_flag == 0)
                continue;
            else if(l_flag == 0)
            {
                hex[hex_i] = hex_h;
                hex_i ++;
                h_flag = 0;
            }
            else
            {
                hex[hex_i] = (hex_h<<4) + hex_l;
                hex_i ++;
                h_flag = 0;
                l_flag =0;
            }
        }
        else
        {
            return hex_i;
        }
    }

    if(h_flag && l_flag == 0)
    {
        hex[hex_i] = hex_h;
        hex_i ++;
    } 
    
    return hex_i;
}

void common_hex2string(unsigned char * hex, char *string, u16 hex_len) 
{ 
    u16 i = 0; 
    string[0] = 0; 
    while (i < hex_len) 
    { 
        sprintf (string + strlen(string), "%02X", (int)hex[i]); 
        i++; 
    } 
}



bool common_isxdigit(char c)
{
    if((c >='0' && c <='9') || (c>='a' && c<='f') ||(c>='A' && c<='F'))
        return TRUE;
    return FALSE;
}

bool common_isdigit(char c)
{
    if(c >='0' && c <='9')
        return TRUE;
    return FALSE;
}

void common_printf_hex(u8 *buf, u16 len)
{
    u16 i;
    for(i=0;i<len;i++)
    {
        DBG_printf(DBG_DEBUG,"%02X ",buf[i]);
        if(i>0)
        if((i+1)%16 == 0)
            DBG_printf(DBG_DEBUG,"\r\n");
        else if((i+1)%8 == 0)
            DBG_printf(DBG_DEBUG," ");
    }
    DBG_printf(DBG_DEBUG,"\r\n");
}


/*
 ˵:
 һַ ضַз
*/
int common_str_span_substr(u8 *str, u8 chSpan,  u8 **section,int maxnum)
{
    int num_i;
    u8 *comma_str;
    for(num_i = 0; num_i<maxnum;)
    {
        comma_str = (u8 *)strchr((const char*)str, (char)chSpan);                    
        if(comma_str != NULL)
            *comma_str = 0;
                
        section[num_i] = str;
        num_i++;
        
        if(comma_str == NULL)
            break;
        str = comma_str+1;  
    }

    return num_i;
}


u8 common_check_ip_empty(u8 *ip)
{
    u8 empty[4] ={0,0,0,0};
    if(memcmp(ip,empty,4)==0)
    {
        return 1; //empty;
    }
    return 0; // not empty
}

